# coding:utf-8

def bubbleSort(nums):
    for i in range(len(nums)-1):    # 这个循环负责设置冒泡排序进行的次数
        for j in range(len(nums)-i-1):  # ｊ为列表下标
            if nums[j] > nums[j+1]:
                nums[j], nums[j+1] = nums[j+1], nums[j]
    return nums
import random
import time
a=[]
for x in range(10000):
    a.append(random.randint(0,10000))
print("生成完成，共10000个随机数，最大值为10000，最小值为0，请稍后...")
tmp=time.time()
c=a[:]
y=time.time()
b=bubbleSort(c)
z=time.time()
n=z-y
print("冒泡排序用时",n,"秒")
def cocktailSort(the_list):
    the_len = len(the_list)
    if the_len <2:#0和1
        return the_list
    else:
        while 1:
            flag = False
            for i in range(the_len-1):
                if the_list[i] > the_list[i+1]:
                    the_list[i], the_list[i+1] = the_list[i+1], the_list[i]
            j = the_len-1
            while j > 0:
                if the_list[j-1] > the_list[j]:
                    the_list[j], the_list[j-1] = the_list[j-1], the_list[j]
                    flag = True
                j -= 1
            if flag == False:
                break
        return the_list
c=a[:]
y=time.time()
b=cocktailSort(c)
z=time.time()
n=z-y
print("鸡尾酒排序用时",n,"秒")
def insertSort(arr):
    length = len(arr)
    for i in range(1,length):
        x = arr[i]
        for j in range(i,-1,-1):
            # j为当前位置，试探j-1位置
            if x < arr[j-1]:
                arr[j] = arr[j-1]
            else:
                # 位置确定为j
                break
        arr[j] = x
    return arr
c=a[:]
y=time.time()
b=insertSort(c)
z=time.time()
n=z-y
print("插入排序用时",n,"秒")
def bucketSort(nums):
  # 选择一个最大的数
  max_num = max(nums)
  # 创建一个元素全是0的列表, 当做桶
  bucket = [0]*(max_num+1)
  # 把所有元素放入桶中, 即把对应元素个数加一
  for i in nums:
    bucket[i] += 1
  # 存储排序好的元素
  sort_nums = []
  # 取出桶中的元素
  for j in range(len(bucket)):
    if bucket[j] != 0:
      for y in range(bucket[j]):
        sort_nums.append(j)
  return sort_nums

c=a[:]
y=time.time()
b=bucketSort(c)
z=time.time()
n=z-y
print("桶排序用时",n,"秒")
def countSort(s):
    min_num = min(s)
    max_num = max(s)
    count_list = [0]*(max_num-min_num+1)
    for i in s:
        count_list[i-min_num] += 1
    s.clear()
    for ind,i in enumerate(count_list):
        while i != 0:
            s.append(ind+min_num)
            i -= 1
    return s
c=a[:]
y=time.time()
b=countSort(c)
z=time.time()
n=z-y
print("计数排序用时",n,"秒")
def merge(arr, l, m, r): 
    n1 = m - l + 1
    n2 = r- m 
  
    # 创建临时数组
    L = [0] * (n1)
    R = [0] * (n2)
  
    # 拷贝数据到临时数组 arrays L[] 和 R[] 
    for i in range(0 , n1): 
        L[i] = arr[l + i] 
  
    for j in range(0 , n2): 
        R[j] = arr[m + 1 + j] 
  
    # 归并临时数组到 arr[l..r] 
    i = 0     # 初始化第一个子数组的索引
    j = 0     # 初始化第二个子数组的索引
    k = l     # 初始归并子数组的索引
  
    while i < n1 and j < n2 : 
        if L[i] <= R[j]: 
            arr[k] = L[i] 
            i += 1
        else: 
            arr[k] = R[j] 
            j += 1
        k += 1
  
    # 拷贝 L[] 的保留元素
    while i < n1: 
        arr[k] = L[i] 
        i += 1
        k += 1
  
    # 拷贝 R[] 的保留元素
    while j < n2: 
        arr[k] = R[j] 
        j += 1
        k += 1
  
def mergeSort(arr,l,r): 
    if l < r: 
  
        
        m = int((l+(r-1))/2)
  
       
        mergeSort(arr, l, m) 
        mergeSort(arr, m+1, r) 
        merge(arr, l, m, r) 
c=a[:]
y=time.time()
b=mergeSort(c,0,len(c)-1)
z=time.time()
n=z-y
print("归并排序用时",n,"秒")
def radix_sort_nums(nums):
    max = nums[0]
    for i in nums:
        if max < i:
            max = i
    times = 0
    while max > 0:
        max = int(max/10)
        times += 1
    return times

# 每个数字各个位置的数字大小，比如（123,1）则是3，（123,2）则是2
def get_num(num,n):
    return (int(num/(10**(n-1)))) % 10

# 主程序
def radixSort(nums):
    count = 10*[None]	# 定义的数组，用于存放当前位数的元素个数
    bucket = len(nums)*[None]	# 用于暂时存放排序结果
	# 分别从个位/十位/百位开始循环
    for pos in range(1, radix_sort_nums(nums)+1):
		# 每次排序完都要清空各个位数存放的数据统计
        for i in range(10):
            count[i] = 0

        for i in range(len(nums)):
			# 获得0到9每个位数的个数
            j = get_num(nums[i], pos)
            count[j] = count[j]+1
		# 获得相对应位数的边界值
        for i in range(1, 10):
            count[i] = count[i] + count[i-1]

        for i in range(len(nums)-1, -1, -1):
			# 求出相应位数的数字
            j = get_num(nums[i], pos)
			#将元素按相应位数的上数字的大小排序
            bucket[count[j]-1] = nums[i]
			#让相应位数上数字相等的元素不会放在同一位置而被替代
            count[j] = count[j]-1

		# 将暂时存储在bucket的元素数据返回到nums中
        for i in range(0, len(nums)):
            nums[i] = bucket[i]
    return nums
c=a[:]
y=time.time()
b=radixSort(c)
z=time.time()
n=z-y
print("基数排序用时",n,"秒")
def gnomeSort(unsorted):
    """
    地精排序，默认升序
    算法思想：定义一个变量i,如果a[i-1]<=a[i],i=i+1,
    否则，则交换，并把i变为i-1,就这样来回移动索引位置，
    是整个列表有序
    这个排序算法是稳定的
    """


    if len(unsorted) <= 1:
        return unsorted
        
    i = 1
    
    while i < len(unsorted):
        if unsorted[i-1] <= unsorted[i]:
            i += 1
        else:
            unsorted[i-1], unsorted[i] = unsorted[i], unsorted[i-1]
            i -= 1
            if (i == 0):
                i = 1

    return unsorted
c=a[:]
y=time.time()
b=gnomeSort(c)
z=time.time()
n=z-y
print("地精排序用时",n,"秒")
print("稳定排序结束！")

print("开始不稳定排序...")
def quick_sort_num(nums,start,end):
    if start < end:
    	# 定义基准值为第一个数
        i, j, pivot = start, end, nums[start]
        while i < j:
        	# 将基准数右边的数中比基准数小的放到左边
            while i < j and nums[j] >= pivot:
                j = j-1
            if i < j:
                nums[i] = nums[j]
                i = i+1
            # 将基准数左边的数中比基准数大的数放到右边
            while i < j and nums[i] < pivot:
                i = i+1
            if i < j:
                nums[j] = nums[i]
                j = j-1
        nums[i] = pivot
        # 分别将基准数左边的数列和右边的数列进行递归
        quick_sort_num(nums, start, i-1)
        quick_sort_num(nums, i+1, end)
    return nums

# 快速排序主体函数
def quickSort(nums):
    start = 0
    end = len(nums)-1
    nums = quick_sort_num(nums, start, end)
    return nums
c=a[:]
y=time.time()
b=quickSort(c)
z=time.time()
n=z-y
print("快速排序用时",n,"秒")
def heapSort(arr):
    # 排序节点
    def node_sort(_arr, _n, _N):
        if _N < 2*_n:
            return
        elif _N == 2*_n:
            if _arr[_n] > _arr[2*_n]:
                _arr[_n], _arr[2*_n] = _arr[2*_n], _arr[_n]
        else:
            min_butree =  2*_n+1 if _arr[2*_n] > _arr[2*_n+1] else 2*_n
            if _arr[_n] > _arr[min_butree]:
                _arr[_n], _arr[min_butree] = _arr[min_butree], _arr[_n]
                node_sort(_arr, min_butree, _N)

    # 初始化得到最大堆
    N = len(arr) - 1
    for i in range(len(arr), 0, -1):
        node_sort(arr, i, N)

    # 堆顶与堆尾交换，下沉后重复得到从大到小排序
    for i in range(N-2):
        arr[1], arr[N-i] = arr[N-i], arr[1]
        node_sort(arr, 1, N-i-1)
    arr[1], arr[2] = arr[2], arr[1]
    return arr
c=a[:]
y=time.time()
b=heapSort(c)
z=time.time()
n=z-y
print("堆排序用时",n,"秒")
from itertools import zip_longest
 
def beadSort(l):
    return list(map(sum, zip_longest(*[[1] * e for e in l], fillvalue=0)))
c=a[:]
y=time.time()
b=beadSort(c)
z=time.time()
n=z-y
print("珠排序用时",n,"秒")

def shellSort(arr): 
  
    n = len(arr)
    gap = int(n/2)
  
    while gap > 0: 
  
        for i in range(gap,n): 
  
            temp = arr[i] 
            j = i 
            while  j >= gap and arr[j-gap] >temp: 
                arr[j] = arr[j-gap] 
                j -= gap 
            arr[j] = temp 
        gap = int(gap/2)
    return arr
c=a[:]
y=time.time()
b=shellSort(c)
z=time.time()
n=z-y
print("希尔排序用时",n,"秒")
cnt=time.time()
print("总共用时",cnt-tmp,"秒")



'''do you want a full list of unicode?
try this:
from Mydemos_pkg.unicodes import unicode_1,unicode_2,BMP_plane2,make_unicode
make_unicode()
unicode_1 #BMP
unicode_2 #plane2
BMP_plane2 #all
'''
print('initilizing...')
unicode_1=[0]*65536
unicode_2=[0]*131072
BMP_plane2=[0]*131072
print('unicodes modules release')
def make_unicode():
    for x in range(65536):
        unicode_1[x]=chr(x)
        BMP_plane2[x]=chr(x)
    for x in range(65536):
        unicode_2[x+65536]=chr(x+65536)
        BMP_plane2[x+65536]=chr(x+65536)
    return None
import unicodedata
del unicodedata



def Help(s=None):
      if s==None:
            Help_ext()
      else:
            Help_main(s)

      return None


def Help_ext(ss=None):
      helpw='''
Welcome to Mydemos_pkg 1.7.0's help utility!

Enter the name of any module, keyword, or topic to get help on writing
Python programs and using Python modules.  To quit this help utility and
return to the interpreter, just type "quit".

To get a list of available modules, type "modules".  Each module also comes with a one-line summary of what it does; to list the modules whose name
or summary contain a given string such as "spam", type "modules spam".
'''
      helpw1='''

help> '''
      hb='''
about bytedesign:
you can output an amazing graph usage:
from Mydemos_pkg import bytedesign
bytedesign.bytedesign()
and look at the turtle-graph!
'''
      hch='''
about chaos:
it make a chaos line to you!
from Mydemos_pkg import chaos
chaos.chaos()
'''
      hcl='''
about clock:
it make a clock to you!
from Mydemos_pkg import clock
clock.clock()
'''

      hco='''
about colormixer:
it make a colormixer to you and u can use your mouse to control it!
from Mydemos_pkg import colormixer
colormixer.colormixer()
'''
      hcs='''
about colorspiral:
it make a colorspiral!
from Mydemos_pkg import colorspiral
colorspiral.colorspiral(6)#sides
'''
      hd='''
about download:
it download a file!
from Mydemos_pkg import download_file
download_file.download(\"http://www.example.com/demo.pdf\")
'''
      hf='''
about forest:
it make a forest!
from Mydemos_pkg import forest
forest.forest() #only_debug
'''
      hfr='''
it makes a curve!
from Mydemos_pkg import fractalcurves
fractalcurves.fractalcurves()
'''
      hh='''
it shows you how hanoi game worked.
from Mydemos_pkg import hanoi
hanoi.hanoi()
'''

      hl='''
it make a lindenmayer.
from Mydemos_pkg import lindenmayer
lindenmayer.lindenmayer()
'''
      hpa='''
it make a paint software.
from Mydemos_pkg import paint
paint.paint()
'''
      hpe='''
it make a peace.
from Mydemos_pkg import peace
peace.peace()
'''
      hpen='''
it make a penrose.
from Mydemos_pkg import penrose
penrose.penrose()
'''
      hpi='''
it could install or uninstall or upgrade sth modules.
usage:
from Mydemos_pkg.pip_install import *
install('paddlehub')
upgrade('paddlehub')
uninstall('paddlehub')
'''
      hpl='''
it make earth,sun,moon.
from Mydemos_pkg import planet_and_moon
planet_and_moon.planet_and_moon()
'''

      hpr='''
it make a progress.
from Mydemos_pkg import progress
progress.progress(0.1)#delay
'''
      hro='''
it make a rosette.
from Mydemos_pkg import penrose
rosette.rosette()
'''
      hr='''
it make a round_dance.
from Mydemos_pkg import round_dance
round_dance.round_dance()
'''
      hs='''
it sort_test.
from Mydemos_pkg import sort
sort.sort_test()
'''
      hsa='''
it sort_test.
from Mydemos_pkg import sort_animate
sort_animate.sort_animate()
'''
      ht='''
it make a tree.
from Mydemos_pkg import tree
tree.tree()
'''
      htw='''
it make two canvas.
from Mydemos_pkg import two_canvas
two_canvas.two_canvas()
'''
      huc='''
it converts unicodes.
nonBMPtoBMP ->unicodes from nonBMP to BMP,if you: print(nonBMPtoBMP('\\U0001F600'))onidle you get a emoji!
'''

      hu='''
make a whole unicode map.
make_unicode ->make unicodemap.
unicode_1 ->BMP
unicode_2 ->Plane2
BMP_plane2 -> BMP and Plane2
'''

      hy='''
it make a yinyang graph.
from Mydemos_pkg import yinyang
yinyang.yinyang()
'''
     #ok
      all_helps={'bytedesign':hb,
                      'chaos':hch,
                      'clock':hcl,
                      'colormixer':hco,
                      'colorspiral':hcs,
                      'download_file':hd,
                      'forest':hf,
                      'fractalcurves':hfr,
                      'hanoi':hh,
                      'lidenmayer':hl,
                      'nim':'nim',
                      'paint':hpa,
                      'peace':hpe,
                      'penrose':hpen,
                      'pip_install':hpi,
                      'planet_and_moon':hpl,
                      'progress':hpr,
                      'rosette':hro,
                      'sort':hs,
                      'sort_animate':hsa,
                      'tree':ht,
                      'two_canvas':htw,
                      'unicodeconvert':huc,
                      'unicodes':hu,
                      'yinyang':hy}


      #print(all_helps)#debug
      mdd='''
bytedesign
chaos
clock
colormixer
colorspiral
download_file
forest
fractalcurves
hanoi
lidenmayer
nim
paint
peace
penrose
pip_install
planet_and_moon
progress
rosette
sort
sort_animate
tree
two_canvas
unicodeconvert
unicodes
yinyang
'''
      if ss==None:
          print(helpw)
          d=input(helpw1)
          while d.lower()!='quit':
                try:
                      print(all_helps[d])
                except KeyError:
                      if d.replace(' ','')=='':
                            sss='''
You are now leaving help and returning to the Python interpreter.
If you want to ask for help on a particular object directly from the
interpreter, you can type "help(object)".  Executing "help('string')"
has the same effect as typing a particular string at the help> prompt.
'''
                            print(sss)
                            return
                      if 'modules' in d:
                            if d.replace(' ','')=='modules':
                                  print(mdd)
                            else:
                                  try:
                                        print(all_helps[d.replace(' ','').replace('modules','')])
                                  except KeyError:
                                        print('No module found as %s'%d)
                      else:
                            print('No module found as %s'%d)
                d=input(helpw1)
          sss='''
You are now leaving help and returning to the Python interpreter.
If you want to ask for help on a particular object directly from the
interpreter, you can type "help(object)".  Executing "help('string')"
has the same effect as typing a particular string at the help> prompt.
'''
          print(sss)
      else:
          try:
              print(all_helps[d])
          except KeyError:
              if d=='modules':
                  print(mdd)
              else:
                  print('No anything found as %s'%ss)


def Help_main(s):
    Help_ext(s)



__doc__='''
do you want ａ to a or a to ａ?try:
HalfToFullWidth('a')
!!!!!!!!!!!!!





'''
HTFW=65248
def HalfToFullWidth(s):
    '''
    This is for like 'ａ'to'a'.
    '''
    try:
        sn=ord(s)+HTFW
        temptp_1_1=chr(sn)
    except:
        raise UnicodeEncodeError("this is not a fullwidth alpha.")
    finally:
        pass
    
    return chr(sn)
def FullToHalfWidth(s):
    '''
    This is for like 'a'to'ａ'.
    '''
    try:
        sn=ord(s)-HTFW
        temptp_1_1=chr(sn)
    except:
        raise UnicodeEncodeError("this is not a fullwidth alpha.")
    finally:
        pass
    return chr(sn)



__doc__='''
A unicodeconvert for demo.
USAGE:
\'\'\'do you want to print idle a smily face(emoji)?maybe u tried this:
print(\'\\U0001F600\')
but you got this:
Traceback (most recent call last):
  File "<pyshell#29>", line 1, in <module>
    print(\'\\U0001F600\')
UnicodeEncodeError: 'UCS-2' codec can't encode character \'\\U0001f600\' in position 0: Non-BMP character not supported in Tk
>>>
\'\'\'
#try this:
nonBMPtoBMP('\U0001F600')
ok,have fun!
'''
class UnicodeConvertError(BaseException):
    pass
def nonBMPtoBMP_e(up):
    u=ord(up)
    vc = u - 0x10000
    vh = (vc & 0xFFC00) >>10
    vl = vc & 0x3FF
    w1 = 0xD800
    w2 = 0xDC00
    w1 = w1|vh
    w2 = w2|vl
    return chr(w1)+chr(w2)
import unicodedata
def UnicodetoASCII(s):
    return unicodedata.normalize('NFKD',s).encode('ascii','ignore')
def ASCIItoUnicode(s):
    return chr(ord(s))
def nonBMPtoBMP(up):
    u=ord(up)
    if u<=0xFFFF:
        raise UnicodeConvertError('The arg shouldn\'t be lesser than 0x10000.')
    else:
        return nonBMPtoBMP_e(up)

def toUTF16(u):
    if u<=0xFFFF:
        return u
    else:
        return nonBMPtoBMP_e(u)




from time import sleep
def progress_s(percent=0, width=30):
    left = width * percent // 100
    right = width - left
    print('\r[', '#' * left, ' ' * right, ']',f' {percent:.0f}%',sep='', end='', flush=True)
def progress(s):
    for i in range(101):
        progress(i)
        sleep(s)



def run(sel):
    if sel=='run':
        pass
    else:
        pass
    global ai_run_ext_status_exit_action_code
    pass
    pass
    a=0
    b=0
    a=c=s=d=f=r=e=g=y=i=0

    if a!=0:
        pass

    return (a,b,c,d,e,f,g,r)


def test(sel):
    try:
        run()
    except Exception as e:
        pass

    return (123456789,45678)

class YouArePigError(BaseException):
    pass
try:
    raise YouArePigError('')
except Exception as e:
    pass
test(sel=1220092)
run()



__doc__='''
This is some demos to learn about turtle, sorting, and something else.
this is only 1.00.1.1 version.
Released in 2020/5/1, the first day of May.
I'll release 2.0.1.0 version in 2020/6/18!
┌───────────────────┐
·demo:bytedesign
 USAGE:
from Mydemos_pkg import bytedesign as bd
bd.bytedesign()
·chaos
·clock
·colormixer
·forest
·fractalcurves
  ...
-------------------------------------------
Anyway, have fun!

'''
name="Mydemos_pkg"



import turtle
print("welcome to colorspiral!")
def colorspiral(sides,bg="black"):
    t = turtle.Pen()
    turtle.bgcolor("black")
    # You can choose between 2 and 6 sides for some cool shapes!,,
    sides = 6
    colors = ["red", "yellow", "blue", "orange", "green", "purple","white"]
    for x in range(360):
        t.pencolor(colors[x%sides])
        t.forward(x * 3/sides + x)
        t.left(360/sides + 1)
        t.width(x*sides/200)



from setuptools import setup
 
NAME = 'Mydemos_pkg'
VERSION = '1.2.7.30'
PACKAGES = ['Mydemos_pkg']
setup(name = NAME
        , version = VERSION
        , packages = PACKAGES,
      description='a small pkg demos for you and everyone',
      long_description='''
Mydemos_pkg1.2.7.0will be a really-new-module:MyHelp!

USAGE just like help()!!!

IfYouSawIt,PleaseInstallIt!

走过路过，不要错过！
```shell
pip install Mydemos_pkg
```
~~lastest_version~~

OK,I will have a list for the version that released.


1.0.1.1

1.1.1.0

1.1.3.0

1.1.4.0

1.2.1.0

1.2.3.0

1.2.3.1


1.2.4.0:added unicode/cs

1.2.5.1:added unicodeerr

1.2.5.9:fixed NameError in unicodeconvert

1.2.6.0b1:may add things in nother way

1.2.6.0:make unicodes

1.2.6.2dev0:download_file.py added

1.2.6.2:added

1.2.7.0dev0 a pre-new module: __**MyHelp**__!!!!!!usage:
```py
from Mydemos_pkg.MyHelp import Help
Help(...)
```
1.2.7.0.debug: __**MyHelp**__ added pre-new command:quit!

1.2.7.0:release Help!

1.2.7.2:release a no-usage module:run!

1.2.7.30:fix progressError
```py
\'\'\'
USAGE
\'\'\'
from Mydemos_pkg.nim import nim
nim()
from Mydemos_pkg.sort import sort_test
sort_test()
```
so it's that.......
_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-

'''
      ,long_description_content_type='text/markdown'
) 



from os import *
def install(s):
    system('python -m pip install '+s+'>pip_install.log')
def uninstall(s):
    system('python -m pip unstall '+s+'>pip_uninstall.log')
def upgrade(s):
    system('python -m pip install --upgrade '+s+'>pip_upgrade.log')
